from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

import sys
import os
import random
import pygame

import mymap
import mytexture
import myfont
import myobjects
import mysmoke

class Game:
	def __init__( self ):
		self.map = None
		self.camx = 4
		self.camy = 10
		self.camz = 40

		#window size
		self.width = 1024	
		self.height = 768

		#animate variables
		self.ux = 0
		self.uy = 0
		self.TimerEvent = USEREVENT + 1		
		self.ground = None
		self.hud = None
		pygame.time.set_timer( self.TimerEvent, 60 );

		self.mapname = "map.txt"

		#hero object
		self.hero = myobjects.Object( .2 )
		self.follow = None
		self.collecting = False
		
		#font object
		self.font = None

		#cop objects
		self.maxcops = 5
		self.cops = []
		self.coptimer = 0

		#wood and cops counter
		self.coptraps = 0
		self.woodcount = 0
		
		#show when you get +10 wood
		self.woodgrowup = []
		self.growuptex = None
		self.growdowntex = None

		#for showing +10 when collected
		self.woodtimer = 0


	def glInit( self, width, height ):
		glViewport( 0, 0, width, height )
		glMatrixMode( GL_PROJECTION )
		glLoadIdentity( )
		gluPerspective( 30, 1.33, 0.1, 400)
		glMatrixMode( GL_MODELVIEW )
		glLoadIdentity( )
		gluLookAt( self.camx, self.camy, self.camz, self.camx, 0, 20, 0, 1, 0 )

	def init( self ):
		#initialization for game
		self.map = mymap.Map( self.mapname )
		self.ground = mytexture.loadTexture( os.path.join("models","grass.tga") )
		self.hud = mytexture.loadTexture( os.path.join("models","hud.tga") )
		self.growuptex = mytexture.loadTexture( os.path.join("models","10.tga") )
		self.growdowntex = mytexture.loadTexture( os.path.join("models","20.tga") )
		self.font = myfont.myFont( os.path.join("models","Font.tga") )
		self.smoke = mysmoke.Smoke( 0, 0, 0, 0, 3, 0, 5, 0.4, os.path.join("models","whitesmoke.tga") );
		self.map.bottomtex = mytexture.loadTexture( os.path.join("models","treebottom.tga") )	
		self.map.traptex = mytexture.loadTexture( os.path.join("models","trap.tga") )
		self.hero.ac3d_obj = self.map.heromodel
		self.follow = self.hero
		(self.hero.x, self.hero.z ) = random.choice( self.map.spawn )
		
		#some initialization for opengl
		glClearColor( 0.0, 0.0, 0.0, 0.0 )
		glEnable( GL_DEPTH_TEST )
		glDepthFunc( GL_LEQUAL )
		glAlphaFunc( GL_GREATER, 0 )
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
		self.glInit( self.width, self.height)

	def restart( self ):
		#map related stuff
		self.map.load(self.mapname)
		self.follow = self.hero
		(self.hero.x, self.hero.z ) = random.choice( self.map.spawn )

		#animation related 
		self.collecting = False
		self.cops = []
		self.coptimer = 0

		#cam movement related
		self.ux = 0
		self.uy = 0

		#game stats related
		self.coptraps = 0
		self.woodcount = 0
	
	def update( self ):		
		#update hero position
		self.hero.update( self.map )

		#update the cam x variable	
		if self.follow == None:
			x = self.camx + self.ux	
			if x > 0 and x < 59:
				self.camx = x
		else:
			self.camx = self.follow.x
	
		y = self.camy + self.uy
		if y > 4 and y < 10 :
			self.camy = y

		#add cops using timer
		self.coptimer += 1
		if self.coptimer >= len(self.cops)*5:
			self.coptimer = 0
			if len(self.cops) < self.maxcops:
				cop = myobjects.Object( )
				(cop.x, cop.z) = random.choice( self.map.spawn )
				cop.ac3d_obj = self.map.copmodel
				cop.findDirection( (self.hero.x, self.hero.z), mymap.TYPE_TRAP )
				#set the spawn position 
				cop.spawn = [ (cop.x, cop.z) ]
				cop.setCopTimer()
				self.cops.append( cop )

		#wood timer (displays 10)
		self.woodtimer += 1
		if self.woodtimer >= 10:
			self.woodtimer = 0
			if self.collecting:
				self.woodcount += 10
				sprite = mysmoke.elem()
				sprite.x = self.map.curtree[0]
				sprite.y = 0
				sprite.z = self.map.curtree[1]
				sprite.texid = self.growuptex
				self.woodgrowup.append( sprite)
				
			if self.hero.objid == mymap.TYPE_TRAP:
				self.woodcount -= 20
				sprite = mysmoke.elem()
				sprite.x = self.hero.x
				sprite.y = 0
				sprite.z = self.hero.z
				sprite.texid = self.growdowntex
				self.woodgrowup.append( sprite)

		#check collision with map
		nlist = []
		for cop in self.cops:
			if cop.coptimer > 0:
				cop.findDirection( (self.hero.x, self.hero.z), mymap.TYPE_TRAP )
			else:
				spawnx = cop.spawn[0][0]
				spawny = cop.spawn[0][1]
				if spawnx -2 < cop.x and cop.x < spawnx+2:
					if spawny -2 < cop.z and cop.z < spawny+2:
						cop.setCopTimer()
				cop.findDirection( cop.spawn[0], mymap.TYPE_TRAP )
				
			cop.update( self.map )
			if cop.alive:
				nlist.append(cop)
			else:
				self.coptraps+=1
		#remove the cops who are not live
		self.cops = nlist

		#for updating the tree angles
		self.map.update( )

		self.smoke.animate(0.04)
		
		#animate the woodgrowuplist
		nlist = []
		minsize = 0.2
		maxsize = .5
		for e in self.woodgrowup:
			e.ts += 0.04
			if e.ts <= 1.0:
				nlist.append( e )			
			e.y = float( 5*e.ts ) 
			e.size = float(minsize + e.ts * maxsize ) 
			e.opacity = (1.0-(e.ts))
		self.woodgrowup = nlist

	def drawRect( self, x1, y1, x2, y2, u1,v1, u2, v2 ):
		glBegin( GL_POLYGON )
		glTexCoord2f( u1, v1 )
		glVertex2f( x1, y1 )
		glTexCoord2f( u2, v1 )
		glVertex2f( x2, y1 )
		glTexCoord2f( u2, v2 )
		glVertex2f( x2, y2 )
		glTexCoord2f( u1, v2 )
		glVertex2f( x1, y2 )
		glEnd( )

	def drawHud( self ):
		glDisable( GL_DEPTH_TEST )
		glEnable( GL_ALPHA_TEST )
		glLoadIdentity( )
		glMatrixMode( GL_PROJECTION )
		glPushMatrix( )
		glLoadIdentity( )
		gluOrtho2D( 0, self.width, 0, self.height )
		
		#start drawing the hud
		glColor3f( 1, 1, 1 )
		if self.hero.objid == mymap.TYPE_TREE or self.hero.objid == mymap.TYPE_COCOTREE:
			self.font.drawString( "HOLD E TO CUT TREE", self.width/2.0-100, 50, 15, 15 )
		elif self.hero.objid == mymap.TYPE_TRAP:
			self.font.drawString( "U LOOSE WOOD IF YOU GO NEAR THE TRAP", self.width/2.0-150, 50, 15, 15 )
		elif self.hero.objid == mymap.TYPE_WALL:
			self.font.drawString( "USE W A S D TO MOVE", self.width/2.0-100, 50, 15, 15 )

		#score panel
		h = self.height
		w = self.width
		hby4 = self.height/4.0
		wby4 = self.width/4.0
		glEnable( GL_TEXTURE_2D )
		glEnable( GL_BLEND )
		glColor4f( 1, 1, 1, 0.8 )
		glBindTexture( GL_TEXTURE_2D, self.hud )
		xmin = w-wby4
		ymin = h-(hby4)
		xmax = w
		ymax = h
		self.drawRect( xmin, ymin, xmax, ymax, 0, 0.31, 1, 0.71 )
		#screw
		self.drawRect( xmin, ymin, xmin+20, ymin+20, 0.257, 0, 0, 0.32 )
		self.drawRect( xmin, ymax-20, xmin+20, ymax, 0.257, 0, 0, 0.32 )
		self.drawRect( xmax-20, ymin, xmax, ymin+20, 0.257, 0, 0, 0.32 )
		self.drawRect( xmax-20, ymax-20, xmax, ymax, 0.257, 0, 0, 0.32 )
		glDisable( GL_BLEND )
		#wood pic
		self.drawRect( xmin+20, ymax-50, xmin+60, ymax-10, 0.257, 0, 0.734, 0.31  )
		#cross mark for wood
		if self.woodcount < self.map.wood:
			self.drawRect( xmax-60, ymax-70, xmax-30, ymax-40, 0.46, 0.73, 0.765, 1 )
		else:
			self.drawRect( xmax-60, ymax-70, xmax-30, ymax-40, 0, 0.73, 0.46, 1 )
		#cop pic
		self.drawRect( xmin+20, ymax-120, xmin+60, ymax-80, 0.734, 0, 1, 0.31  )
		#cross mark for cop 
		if self.coptraps < self.map.cops:
			self.drawRect( xmax-60, ymax-140, xmax-30, ymax-110, 0.46, 0.73, 0.765, 1 )
		else:
			self.drawRect( xmax-60, ymax-140, xmax-30, ymax-110, 0, 0.73, 0.46, 1 )
		
		glDisable( GL_TEXTURE_2D )

		#Text
		glColor3f( 1, .8, .1 )
		self.font.drawString( "WOOD", xmin+80, ymax-20, 15, 15 )
		self.font.drawString( "TARGET:", xmin+60, ymax-50, 10, 10 )
		self.font.drawString( str(self.map.wood), xmin+150, ymax-50, 10, 10 )
		self.font.drawString( "CURRENT:", xmin+60, ymax-70, 10, 10 )
		self.font.drawString( str(self.woodcount), xmin+150, ymax-70, 10, 10 )
		self.font.drawString( "COPS", xmin+80, ymax-90, 15, 15 )
		self.font.drawString( "TARGET:", xmin+60, ymax-120, 10, 10 )
		self.font.drawString( str(self.map.cops), xmin+150, ymax-120, 10, 10 )
		self.font.drawString( "CURRENT:", xmin+60, ymax-140, 10, 10 )
		self.font.drawString( str(self.coptraps), xmin+150, ymax-140, 10, 10 )
	

		

		#end drawing hud
		glPopMatrix( )
		glEnable( GL_DEPTH_TEST )
		glDisable( GL_ALPHA_TEST )

	def display( self ):

		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )
		glMatrixMode( GL_MODELVIEW )
		glLoadIdentity( )
		gluLookAt( self.camx, self.camy, self.camz, self.camx, 0, 20, 0, 1, 0 )	
				
		#drawing the ground
		glColor3f( 1, 1, 1 )
		glEnable( GL_TEXTURE_2D )
		glBindTexture( GL_TEXTURE_2D, self.ground )
		glBegin( GL_POLYGON )
		glTexCoord2f( 0, 0 )
		glVertex3f( 0, 0, 0 ) 
		glTexCoord2f( 6, 0 )
		glVertex3f( 60, 0, 0 )
		glTexCoord2f( 6, 3 )
		glVertex3f( 60, 0, 30 )
		glTexCoord2f( 0, 3 )
		glVertex3f( 0, 0, 30 )
		glEnd()
		glDisable( GL_TEXTURE_2D )

		#render the contents of map
		self.map.render( self.camx )
		
		#render the hero of game
		self.hero.render( )

		#render the cops in game
		cx = int( self.camx )
		for cop in self.cops:
			if cx-10 < int(cop.x) and int(cop.x) < cx+10:
				cop.render( )
		#draw the numbers
		for e in self.woodgrowup:
			glEnable( GL_BLEND )
			glEnable( GL_ALPHA_TEST )
			glEnable( GL_TEXTURE_2D )
			glDepthMask(0)
			glBindTexture( GL_TEXTURE_2D, e.texid )
			glColor4f( 1, 1, 1, e.opacity )
			glBegin(GL_QUADS);
			glTexCoord2f(0, 0); 
			glVertex3f(-1*e.size+e.x,-1*e.size+e.y, -0.+e.z);
			glTexCoord2f(0, 1); 
			glVertex3f(-1*e.size+e.x, 1*e.size+e.y,  0.+e.z);
			glTexCoord2f(1, 1);	
			glVertex3f( 1.*e.size+e.x, 1*e.size+e.y,  0.+e.z);
			glTexCoord2f(1, 0);	
			glVertex3f( 1.*e.size+e.x,-1.*e.size+e.y, -0.+e.z);
			glEnd();
			glDepthMask(1)
			glDisable( GL_ALPHA_TEST )
			glDisable( GL_BLEND )
		
		glPushMatrix()
		glColor3f( 1, 1, 1 );
		glTranslatef( self.hero.x, 1.5, self.hero.z )
		self.smoke.render( )
		glPopMatrix()

		self.drawHud( )

		glFlush( )
		pygame.display.flip( )

	def procEvents( self, event ):
			
		if event.type == KEYDOWN:
			#camera movement keys
			if event.key == K_ESCAPE:
				sys.exit(0)
			elif event.key == K_RIGHT:
				self.ux = 0.5
			elif event.key == K_LEFT:
				self.ux = -0.5	
			elif event.key == K_PAGEDOWN:
				self.uy = -0.5
			elif event.key == K_PAGEUP:
				self.uy = 0.5
				
			#player movement keys
			if event.key == K_w:
				self.hero.moveTop( self.map )
			elif event.key == K_s:
				self.hero.moveBack( self.map )
			elif event.key == K_a:
				self.hero.moveLeft( self.map )
			elif event.key == K_d:
				self.hero.moveRight( self.map )
			#cop trap key  
			elif event.key == K_SPACE:
				self.map.addTrap( int(self.hero.x), int(self.hero.z) )
				#reducing the wood and adding sprite
				sprite = mysmoke.elem()
				sprite.x = int(self.hero.x)
				sprite.y = 0
				sprite.z = int(self.hero.z)
				sprite.texid = self.growdowntex
				self.woodgrowup.append( sprite)
				self.woodcount -= 20

			elif event.key == K_e:
				#User has typed E for cutting the tree.
				if self.hero.objid == mymap.TYPE_TREE or self.hero.objid == mymap.TYPE_COCOTREE:
					x = int(self.hero.x)
					y = int(self.hero.z)
					self.map.startAnimation( )
					self.collecting = True
					#user started cutting Tree
					if self.hero.prevdir == myobjects.LEFT:
						self.map.curtree= [ x-1, y ] 
						#rotate about z
						self.map.setAxis([ 0, 0, 1 ])
						print "Tree is to Left"
					elif self.hero.prevdir == myobjects.RIGHT:
						self.map.curtree= [ x+1, y ] 
						#rotate about z
						self.map.setAxis( [ 0, 0, 1 ] )
						print "Tree is to Right"
					elif self.hero.prevdir == myobjects.TOP:
						self.map.curtree= [ x, y-1 ] 
						#rotate about x
						self.map.setAxis( [ 1, 0, 0 ] )
						print "Tree is to Top"
					elif self.hero.prevdir == myobjects.BACK:
						self.map.curtree= [ x, y+1 ] 
						#rotate about x
						self.map.setAxis( [ 1, 0, 0 ] )
						print "Tree is to Back"

			#special key to set follow
			if event.key == K_h:
				if self.follow == None:
					self.follow = self.hero
				else:
					self.follow = None

		if event.type == self.TimerEvent:
			self.update( )
			
		if event.type == KEYUP:
			if event.key == K_RIGHT or event.key == K_LEFT:
				self.ux = 0
			if event.key == K_PAGEUP or event.key == K_PAGEDOWN:
				self.uy = 0
			#hero keys
			if event.key == K_e:
				#stop animating tree	
				self.map.stopAnimation( )
				self.collecting = False

		if event.type is MOUSEMOTION:
			(x,y) =  event.pos
			if( x >= self.width - 10 ):
				self.ux = 0.5
			elif( x < 10 ):
				self.ux = -0.5
			else:
				self.ux = 0

		if event.type is MOUSEBUTTONDOWN:
			(x,y) = event.pos
			print x,y 

if __name__ == "__main__":
	pygame.init( )
	game = Game( )
	pygame.display.set_mode( (game.width, game.height), OPENGL|DOUBLEBUF)
	game.init( )
	while 1:
		event = pygame.event.poll( )
		if event.type is QUIT:
			sys.exit(0)
		game.display( )
		game.procEvents( event )
