import os
import sys
from OpenGL.GL import *
from OpenGL.GLU import *

import myobjects
import myac3d


#type specification
TYPE_TREE = 1
TYPE_WALL = 2	
TYPE_SPAWN = 3
TYPE_DESTINATION = 4
TYPE_COCOTREE = 5
TYPE_GRASS = 6
TYPE_WALL1 = 7 
TYPE_TRAP = -1

class Map:
	def __init__( self, filename ):
		if filename=='None':
			sys.exit(1)
		self.spawn = []	#spawn positions for cops as well as hero
		self.texture = None	#holds the texture of ground
		self.objects = []	#list of lists of objects at each strip
		self.wood = 0	#wood to be collected
		self.cops = 0	#cops to be trapped

		self.cocotree = myac3d.ACObject( )
		self.cocotree.load( os.path.join("models","cocotree.ac") )

		self.treemodel = myac3d.ACObject( )
		self.treemodel.load( os.path.join("models","tree.ac") )

		self.grassmodel = myac3d.ACObject( )
		self.grassmodel.load( os.path.join("models","modelgrass.ac") )

		self.wallmodel = myac3d.ACObject( )
		self.wallmodel.load( os.path.join("models","wall.ac") )

		self.copmodel = myac3d.ACObject( )
		self.copmodel.load( os.path.join("models","cop.ac") )

		self.heromodel = myac3d.ACObject( )
		self.heromodel.load( os.path.join("models","woodcutter.ac") )


		#object textures (those rendered here directly)
		self.bottomtex = None
		self.traptex = None

		#animation of tree
		self.curtree = []
		self.axis = []
		self.rangle = 0
		self.rupdate = 0.4
		self.animate = False
#		self.treehealth = 100

		self.load( filename )		
	
	def stopAnimation( self ):
		self.animate = False
		self.curtree = []
		self.axis = []
#		self.rupdate = 0

	def startAnimation( self ):
		self.animate = True
#		self.rupdate = 0.4
#		self.treehealth = 100
		
	def setAxis( self, axis ):
		self.axis = axis
	
	def load( self, filename ):				

		#animation of tree put here in case of map.load
		self.curtree = []
		self.axis = []
		self.rangle = 0
		self.rupdate = 0.4
		self.animate = False
		
		fp = open( filename, "r" )
		
		#read the wood to be cut
		line = fp.readline( )
		(token,val) = line.split( )
		self.wood = int(val)

		#read the cops to be ditched
		line = fp.readline()
		(token,val) = line.split( )
		self.cops = int(val)
		
		#read the number of spawns
		line = fp.readline( )
		(token,val) = line.split( )
		for i in range(int(val)):
			line = fp.readline( )
			(v1,v2) = line.split( )
			self.spawn.append( (int(v1),int(v2)) )

		#read the objects in map
		line = fp.readline( )
		xextent = int( line )
		line = fp.readline( )
		yextent = int( line )
		
		for i in range( xextent ):
			self.objects.append( [] )
			line = fp.readline( )
			self.objects[i] = line.split( )
			nlist = map( int, self.objects[i])
			self.objects[i] = nlist
		fp.close( )

	def update( self ):
		#for animating the tree while being cut
		if self.animate:
			#self.treehealth -= 10
			self.rangle += self.rupdate
			#if self.treehealth > 0 :
			if self.rangle < -2 or self.rangle>2:
				self.rupdate = -1*self.rupdate
			#else :
			#	self.rupdate = 1
			#	if self.rangle <= -90 or self.rangle >= 90:
			#		self.objects[self.curtree[0]][self.curtree[1]] = 0
			#		self.stopAnimation( )

	def addTrap( self, i, j ):
		self.objects[i][j] = TYPE_TRAP
		
	def renderTree( self, i, j, type ):
		glPushMatrix( )
		glTranslatef( i, 0, j )
		glRotatef( (i+j)*45, 0, 1, 0 )
		glColor3f( 1, 1, 1 )
		
		glEnable( GL_ALPHA_TEST )
		glEnable( GL_TEXTURE_2D )
		glColor4f( 1, 1, 1, 0.8 )
		glEnable( GL_BLEND )
		glBindTexture( GL_TEXTURE_2D, self.bottomtex )
		glBegin( GL_POLYGON )
		glTexCoord2f( 0, 0 )
		glVertex3f( -1, 0.001, -1 )
		glTexCoord2f( 1, 0 )
		glVertex3f( 1, 0.001, -1 )
		glTexCoord2f( 1, 1 )
		glVertex3f( 1, 0.001, 1 )
		glTexCoord2f( 0, 1 )
		glVertex3f( -1, 0.001, 1 )
		glEnd( )
		glDisable( GL_BLEND )
		glDisable( GL_TEXTURE_2D )
		glDisable( GL_ALPHA_TEST)
		
		glScalef( 1.5, 1.5, 1.5 )
		if self.animate and self.curtree[0]==i and self.curtree[1]==j:
			glRotatef( self.rangle, self.axis[0], self.axis[1], self.axis[2] )
		if type == TYPE_TREE:
			self.treemodel.render( )
		else:
			self.cocotree.render( )
		glPopMatrix( )

	def renderGrass( self, i, j ):
		glPushMatrix( )
		glTranslatef( i, 0, j )
		glRotatef( (i+j)*45, 0, 1, 0 )
		glColor3f( 1, 1, 1 )
		self.grassmodel.render( )
		glPopMatrix( )


	def renderWall( self, i, j, rotate):
		glColor3f( 1, 1, 1 )
		glEnable( GL_ALPHA_TEST )
		glPushMatrix( )
		glTranslatef( i, 0, j )
		if rotate:
			glRotatef( 90, 0, 1, 0 )
		self.wallmodel.render( )
		glPopMatrix( )	
		glDisable( GL_ALPHA_TEST )

	def renderTrap( self, i, j ):
		glColor3f( 1, 1, 1 )
		glEnable( GL_TEXTURE_2D )
		glEnable( GL_ALPHA_TEST )
		glBindTexture( GL_TEXTURE_2D, self.traptex )
		glBegin( GL_POLYGON )
		glTexCoord2f( 0, 0 )
		glVertex3f( -1+i, 0.001, -1+j )
		glTexCoord2f( 1, 0 )
		glVertex3f( 1+i, 0.001, -1+j )
		glTexCoord2f( 1, 1 )
		glVertex3f( 1+i, 0.001, 1+j )
		glTexCoord2f( 0, 1 )
		glVertex3f( -1+i, 0.001, 1+j )
		glEnd( )
		glDisable( GL_ALPHA_TEST )
		glDisable( GL_TEXTURE_2D )
			
	def render( self, cam_x ):
		global TYPE_TREE
		global TYPE_WALL1
		global TYPE_GRASS
		global TYPE_WALL
		global TYPE_SPAWN
		global TYPE_DESTINATION
		global TYPE_TRAP

		x_index = int(cam_x)
		if x_index <= 10:
			x_index = 10
		if x_index >= 50:
			x_index = 50
		for i in range( x_index-10, x_index+10 ):
			for j in range( 0, 15):
				curobj = self.objects[i][j]
				if curobj == TYPE_TREE or curobj == TYPE_COCOTREE:
					self.renderTree( i, j, curobj )
				elif curobj == TYPE_WALL:
					self.renderWall( i, j, False )
				elif curobj == TYPE_WALL1:
					self.renderWall( i, j, True )
				elif curobj == TYPE_TRAP:
					self.renderTrap( i, j )
				elif curobj == TYPE_GRASS:
					self.renderGrass( i, j )
				elif curobj == TYPE_DESTINATION:
					pass

		for i in range( x_index-7, x_index+8 ):
			for j in range( 15, len(self.objects[i])):
				curobj = self.objects[i][j]
				if curobj == TYPE_TREE or curobj == TYPE_COCOTREE:
					self.renderTree( i, j, curobj )
				elif curobj == TYPE_WALL:
					self.renderWall( i, j, False )
				elif curobj == TYPE_WALL1:
					self.renderWall( i, j, True )
				elif curobj == TYPE_TRAP:
					self.renderTrap( i, j )
				elif curobj == TYPE_GRASS:
					self.renderGrass( i, j )
				elif curobj == TYPE_DESTINATION:
					pass
					
