from OpenGL.GL import *
from OpenGL.GLU import *

import math
import random

#defines for directions
LEFT = 1
RIGHT = 2
TOP = 3
BACK = 4

class Object:
	def __init__( self, speed=0.1, acObj=None ):
		self.x = 0
		self.z = 0
		self.direction = None
		self.ac3d_obj = acObj
		self.speed = speed
		#for knowing the object location and type after collision
		self.prevdir = None
		self.objid = 0
		self.alive = True

		#for cops specifically
		self.coptimer = 0
		self.spawn = []

	def checkDistance( self, hx, hz, dir ):
		xval = 0
		yval = 0
		if dir == RIGHT:
			xval = self.x+1 - hx  
			yval = self.z - hz
		elif dir == LEFT:
			xval = self.x-1 - hx
			yval = self.z - hz
		elif dir == TOP:
			xval = self.x - hx
			yval = self.z-1-hz
		elif dir == BACK:
			xval = self.x - hx
			yval = self.z+1-hz
#		return math.sqrt(xval*xval+yval*yval)
		return  math.fabs(xval)+math.fabs(yval) 
	
	def isTrapped( self, TYPE_TRAP ):
		#bot is trapped
		if self.objid == TYPE_TRAP:
			self.alive = False
		return not(self.alive)
			
		
	def findDirection( self, pos, TYPE_TRAP ):
		
		if self.isTrapped( TYPE_TRAP ):
			return

		#reduce the time
		self.coptimer-=1

		hx = int( pos[0] )
		hz = int( pos[1] )
		#find distance from current point to the hero point
		#choose among the best one
		choices = [LEFT, RIGHT, BACK, TOP]
		#remove the choice which has already been used
		if not(self.prevdir == None): 
			choices.remove( self.prevdir )	

		min = 1000
		for choice in choices:
			val = self.checkDistance(  hx, hz, choice )
			if val < min :
				self.direction = choice
				min = val

	def resetPrevData( self ):
		self.prevdir = None
		self.objid = 0

	def setCopTimer( self ):
		self.coptimer = random.choice([ 500, 350, 600 ])
		
	def update( self, map ):
		if self.direction == LEFT:
			self.moveLeft( map )
		elif self.direction == RIGHT:
			self.moveRight( map )
		elif self.direction == TOP:
			self.moveTop( map )
		elif self.direction == BACK:
			self.moveBack( map )

	def moveRight( self, map ):
		x = int(self.x)# + self.speed)
		y = int(self.z)
		if( x>=0 and x+1 < len(map.objects)  and y>=0 and y < len(map.objects[0]) ):
			if map.objects[x+1][y] == 0:
				self.direction = RIGHT			
				self.x += self.speed
				self.resetPrevData( )
			else:
				self.prevdir = RIGHT
				self.objid = map.objects[x+1][y]
				self.direction = None
		else:
			self.direction = None

	def moveLeft( self, map ):
		x = int(self.x)# - self.speed)
		y = int(self.z)
		if( x>0 and x < len(map.objects)  and y>=0 and y < len(map.objects[0]) ):
			if map.objects[x-1][y] == 0:
				self.direction = LEFT			
				self.x -= self.speed
				self.resetPrevData( )
			else:
				self.prevdir = LEFT
				self.objid = map.objects[x-1][y]
				self.direction = None
		else:
			self.direction = None

	def moveTop( self, map ):
		x = int(self.x)
		y = int(self.z)#+self.speed)
		if( x>=0 and x < len(map.objects)  and y>0 and y < len(map.objects[0]) ):
			if map.objects[x][y-1] == 0:
				self.direction = TOP			
				self.z -= self.speed
				self.resetPrevData( )				
			else:
				self.prevdir = TOP
				self.objid = map.objects[x][y-1]
				self.direction = None
		else:
			self.direction = None		

	def moveBack( self, map ):
		x = int(self.x)
		y = int(self.z)#-self.speed)
		if( x>=0 and x < len(map.objects)  and y>=0 and y+1 < len(map.objects[0]) ):
			if map.objects[x][y+1] == 0:
				self.direction = BACK			
				self.z += self.speed
				self.resetPrevData( )
			else:
				self.prevdir = BACK
				self.objid = map.objects[x][y+1]
				self.direction = None
		else:
			self.direction = None		

	def setRotation( self, direction ):
		if direction == LEFT:
			glRotatef( -90, 0, 1, 0 )
		elif direction == RIGHT:
			glRotatef( 90, 0, 1, 0 )
		elif direction == TOP:
			glRotatef( -180, 0, 1, 0 )

	def render( self ):
		glPushMatrix( )
		glTranslatef( self.x, 0, self.z )
		#default bot faces BACK
		if self.direction == None:
			self.setRotation( self.prevdir )
		else:
			self.setRotation( self.direction )
		glScalef( 1.5, 1.5, 1.5 )
		self.ac3d_obj.render( )
		glPopMatrix( )
